//+------------------------------------------------------------------+
//|                                                   errorcodes.mq4 |
//|                                   Copyright  2008, Adam Parusel |
//|                                                http://esolvix.de |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, Adam Parusel"
#property link      "http://esolvix.de"

//---- errors returned from trade server
#define ERR_NO_ERROR                                  0
#define ERR_NO_RESULT                                 1
#define ERR_COMMON_ERROR                              2
#define ERR_INVALID_TRADE_PARAMETERS                  3
#define ERR_SERVER_BUSY                               4
#define ERR_OLD_VERSION                               5
#define ERR_NO_CONNECTION                             6
#define ERR_NOT_ENOUGH_RIGHTS                         7
#define ERR_TOO_FREQUENT_REQUESTS                     8
#define ERR_MALFUNCTIONAL_TRADE                       9
#define ERR_ACCOUNT_DISABLED                         64
#define ERR_INVALID_ACCOUNT                          65
#define ERR_TRADE_TIMEOUT                           128
#define ERR_INVALID_PRICE                           129
#define ERR_INVALID_STOPS                           130
#define ERR_INVALID_TRADE_VOLUME                    131
#define ERR_MARKET_CLOSED                           132
#define ERR_TRADE_DISABLED                          133
#define ERR_NOT_ENOUGH_MONEY                        134
#define ERR_PRICE_CHANGED                           135
#define ERR_OFF_QUOTES                              136
#define ERR_BROKER_BUSY                             137
#define ERR_REQUOTE                                 138
#define ERR_ORDER_LOCKED                            139
#define ERR_LONG_POSITIONS_ONLY_ALLOWED             140
#define ERR_TOO_MANY_REQUESTS                       141
#define ERR_TRADE_MODIFY_DENIED                     145
#define ERR_TRADE_CONTEXT_BUSY                      146
#define ERR_TRADE_EXPIRATION_DENIED                 147
#define ERR_TRADE_TOO_MANY_ORDERS                   148
//---- mql4 run time errors
#define ERR_NO_MQLERROR                            4000
#define ERR_WRONG_FUNCTION_POINTER                 4001
#define ERR_ARRAY_INDEX_OUT_OF_RANGE               4002
#define ERR_NO_MEMORY_FOR_CALL_STACK               4003
#define ERR_RECURSIVE_STACK_OVERFLOW               4004
#define ERR_NOT_ENOUGH_STACK_FOR_PARAM             4005
#define ERR_NO_MEMORY_FOR_PARAM_STRING             4006
#define ERR_NO_MEMORY_FOR_TEMP_STRING              4007
#define ERR_NOT_INITIALIZED_STRING                 4008
#define ERR_NOT_INITIALIZED_ARRAYSTRING            4009
#define ERR_NO_MEMORY_FOR_ARRAYSTRING              4010
#define ERR_TOO_LONG_STRING                        4011
#define ERR_REMAINDER_FROM_ZERO_DIVIDE             4012
#define ERR_ZERO_DIVIDE                            4013
#define ERR_UNKNOWN_COMMAND                        4014
#define ERR_WRONG_JUMP                             4015
#define ERR_NOT_INITIALIZED_ARRAY                  4016
#define ERR_DLL_CALLS_NOT_ALLOWED                  4017
#define ERR_CANNOT_LOAD_LIBRARY                    4018
#define ERR_CANNOT_CALL_FUNCTION                   4019
#define ERR_EXTERNAL_CALLS_NOT_ALLOWED             4020
#define ERR_NO_MEMORY_FOR_RETURNED_STR             4021
#define ERR_SYSTEM_BUSY                            4022
#define ERR_INVALID_FUNCTION_PARAMSCNT             4050
#define ERR_INVALID_FUNCTION_PARAMVALUE            4051
#define ERR_STRING_FUNCTION_INTERNAL               4052
#define ERR_SOME_ARRAY_ERROR                       4053
#define ERR_INCORRECT_SERIESARRAY_USING            4054
#define ERR_CUSTOM_INDICATOR_ERROR                 4055
#define ERR_INCOMPATIBLE_ARRAYS                    4056
#define ERR_GLOBAL_VARIABLES_PROCESSING            4057
#define ERR_GLOBAL_VARIABLE_NOT_FOUND              4058
#define ERR_FUNC_NOT_ALLOWED_IN_TESTING            4059
#define ERR_FUNCTION_NOT_CONFIRMED                 4060
#define ERR_SEND_MAIL_ERROR                        4061
#define ERR_STRING_PARAMETER_EXPECTED              4062
#define ERR_INTEGER_PARAMETER_EXPECTED             4063
#define ERR_DOUBLE_PARAMETER_EXPECTED              4064
#define ERR_ARRAY_AS_PARAMETER_EXPECTED            4065
#define ERR_HISTORY_WILL_UPDATED                   4066
#define ERR_TRADE_ERROR                            4067
#define ERR_END_OF_FILE                            4099
#define ERR_SOME_FILE_ERROR                        4100
#define ERR_WRONG_FILE_NAME                        4101
#define ERR_TOO_MANY_OPENED_FILES                  4102
#define ERR_CANNOT_OPEN_FILE                       4103
#define ERR_INCOMPATIBLE_FILEACCESS                4104
#define ERR_NO_ORDER_SELECTED                      4105
#define ERR_UNKNOWN_SYMBOL                         4106
#define ERR_INVALID_PRICE_PARAM                    4107
#define ERR_INVALID_TICKET                         4108
#define ERR_TRADE_NOT_ALLOWED                      4109
#define ERR_LONGS_NOT_ALLOWED                      4110
#define ERR_SHORTS_NOT_ALLOWED                     4111
#define ERR_OBJECT_ALREADY_EXISTS                  4200
#define ERR_UNKNOWN_OBJECT_PROPERTY                4201
#define ERR_OBJECT_DOES_NOT_EXIST                  4202
#define ERR_UNKNOWN_OBJECT_TYPE                    4203
#define ERR_NO_OBJECT_NAME                         4204
#define ERR_OBJECT_COORDINATES_ERROR               4205
#define ERR_NO_SPECIFIED_SUBWINDOW                 4206
#define ERR_SOME_OBJECT_ERROR                      4207

string ErrorDescription(int code)
{
   switch(code)
   {
      case ERR_NO_ERROR                               : Print("No error returned"); break;
      case ERR_NO_RESULT                              : Print("No error returned, but the result is unknown"); break;
      case ERR_COMMON_ERROR                           : Print("Common error"); break;
      case ERR_INVALID_TRADE_PARAMETERS               : Print("Invalid trade parameters"); break;
      case ERR_SERVER_BUSY                            : Print("Trade server is busy"); break;
      case ERR_OLD_VERSION                            : Print("Old version of the client terminal"); break;
      case ERR_NO_CONNECTION                          : Print("No connection with trade server"); break;
      case ERR_NOT_ENOUGH_RIGHTS                      : Print("Not enough rights"); break;
      case ERR_TOO_FREQUENT_REQUESTS                  : Print("Too frequent requests"); break;
      case ERR_MALFUNCTIONAL_TRADE                    : Print("Malfunctional trade operation"); break;
      case ERR_ACCOUNT_DISABLED                       : Print("Account disabled"); break;
      case ERR_INVALID_ACCOUNT                        : Print("Invalid account"); break;
      case ERR_TRADE_TIMEOUT                          : Print("Trade timeout"); break;
      case ERR_INVALID_PRICE                          : Print("Invalid price"); break;
      case ERR_INVALID_STOPS                          : Print("Invalid stops"); break;
      case ERR_INVALID_TRADE_VOLUME                   : Print("Invalid trade volume"); break;
      case ERR_MARKET_CLOSED                          : Print("Market is closed"); break;
      case ERR_TRADE_DISABLED                         : Print("Trade is disabled"); break;
      case ERR_NOT_ENOUGH_MONEY                       : Print("Not enough money"); break;
      case ERR_PRICE_CHANGED                          : Print("Price changed"); break;
      case ERR_OFF_QUOTES                             : Print("Off quotes"); break;
      case ERR_BROKER_BUSY                            : Print("Broker is busy"); break;
      case ERR_REQUOTE                                : Print("Requote"); break;
      case ERR_ORDER_LOCKED                           : Print("Order is locked"); break;
      case ERR_LONG_POSITIONS_ONLY_ALLOWED            : Print("Long positions only allowed"); break;
      case ERR_TOO_MANY_REQUESTS                      : Print("Too many requests"); break;
      case ERR_TRADE_MODIFY_DENIED                    : Print("Modification denied because order too close to market"); break;
      case ERR_TRADE_CONTEXT_BUSY                     : Print("Trade context is busy"); break;
      case ERR_TRADE_EXPIRATION_DENIED                : Print("Expirations are denied by broker"); break;
      case ERR_TRADE_TOO_MANY_ORDERS                  : Print("The amount of open and pending orders has reached the limit set by the broker."); break;
      case ERR_NO_MQLERROR                            : Print("No error"); break;
      case ERR_WRONG_FUNCTION_POINTER                 : Print("Wrong function pointer"); break;
      case ERR_ARRAY_INDEX_OUT_OF_RANGE               : Print("Array index is out of range"); break;
      case ERR_NO_MEMORY_FOR_CALL_STACK               : Print("No memory for function call stack"); break;
      case ERR_RECURSIVE_STACK_OVERFLOW               : Print("Recursive stack overflow"); break;
      case ERR_NOT_ENOUGH_STACK_FOR_PARAM             : Print("Not enough stack for parameter"); break;
      case ERR_NO_MEMORY_FOR_PARAM_STRING             : Print("No memory for parameter string"); break;
      case ERR_NO_MEMORY_FOR_TEMP_STRING              : Print("No memory for temp string"); break;
      case ERR_NOT_INITIALIZED_STRING                 : Print("Not initialized string"); break;
      case ERR_NOT_INITIALIZED_ARRAYSTRING            : Print("Not initialized string in array"); break;
      case ERR_NO_MEMORY_FOR_ARRAYSTRING              : Print("No memory for array string"); break;
      case ERR_TOO_LONG_STRING                        : Print("Too long string"); break;
      case ERR_REMAINDER_FROM_ZERO_DIVIDE             : Print("Remainder from zero divide"); break;
      case ERR_ZERO_DIVIDE                            : Print("Zero divide"); break;
      case ERR_UNKNOWN_COMMAND                        : Print("Unknown command"); break;
      case ERR_WRONG_JUMP                             : Print("Wrong jump (never generated error)"); break;
      case ERR_NOT_INITIALIZED_ARRAY                  : Print("Not initialized array"); break;
      case ERR_DLL_CALLS_NOT_ALLOWED                  : Print("DLL calls are not allowed"); break;
      case ERR_CANNOT_LOAD_LIBRARY                    : Print("Cannot load library"); break;
      case ERR_CANNOT_CALL_FUNCTION                   : Print("Cannot call function"); break;
      case ERR_EXTERNAL_CALLS_NOT_ALLOWED             : Print("Expert function calls are not allowed"); break;
      case ERR_NO_MEMORY_FOR_RETURNED_STR             : Print("Not enough memory for temp string returned from function"); break;
      case ERR_SYSTEM_BUSY                            : Print("System is busy (never generated error)"); break;
      case ERR_INVALID_FUNCTION_PARAMSCNT             : Print("Invalid function parameters count"); break;
      case ERR_INVALID_FUNCTION_PARAMVALUE            : Print("Invalid function parameter value"); break;
      case ERR_STRING_FUNCTION_INTERNAL               : Print("String function internal error"); break;
      case ERR_SOME_ARRAY_ERROR                       : Print("Some array error"); break;
      case ERR_INCORRECT_SERIESARRAY_USING            : Print("Incorrect series array using"); break;
      case ERR_CUSTOM_INDICATOR_ERROR                 : Print("Custom indicator error"); break;
      case ERR_INCOMPATIBLE_ARRAYS                    : Print("Arrays are incompatible"); break;
      case ERR_GLOBAL_VARIABLES_PROCESSING            : Print("Global variables processing error"); break;
      case ERR_GLOBAL_VARIABLE_NOT_FOUND              : Print("Global variable not found"); break;
      case ERR_FUNC_NOT_ALLOWED_IN_TESTING            : Print("Function is not allowed in testing mode"); break;
      case ERR_FUNCTION_NOT_CONFIRMED                 : Print("Function is not confirmed"); break;
      case ERR_SEND_MAIL_ERROR                        : Print("Send mail error"); break;
      case ERR_STRING_PARAMETER_EXPECTED              : Print("String parameter expected"); break;
      case ERR_INTEGER_PARAMETER_EXPECTED             : Print("Integer parameter expected"); break;
      case ERR_DOUBLE_PARAMETER_EXPECTED              : Print("Double parameter expected"); break;
      case ERR_ARRAY_AS_PARAMETER_EXPECTED            : Print("Array as parameter expected"); break;
      case ERR_HISTORY_WILL_UPDATED                   : Print("Requested history data in updating state"); break;
      case ERR_TRADE_ERROR                            : Print("Some error in trading function"); break;
      case ERR_END_OF_FILE                            : Print("End of file"); break;
      case ERR_SOME_FILE_ERROR                        : Print("Some file error"); break;
      case ERR_WRONG_FILE_NAME                        : Print("Wrong file name"); break;
      case ERR_TOO_MANY_OPENED_FILES                  : Print("Too many opened files"); break;
      case ERR_CANNOT_OPEN_FILE                       : Print("Cannot open file"); break;
      case ERR_INCOMPATIBLE_FILEACCESS                : Print("Incompatible access to a file"); break;
      case ERR_NO_ORDER_SELECTED                      : Print("No order selected"); break;
      case ERR_UNKNOWN_SYMBOL                         : Print("Unknown symbol"); break;
      case ERR_INVALID_PRICE_PARAM                    : Print("Invalid price"); break;
      case ERR_INVALID_TICKET                         : Print("Invalid ticket"); break;
      case ERR_TRADE_NOT_ALLOWED                      : Print("Trade is not allowed - Enable checkbox -Allow live trading- in the expert properties"); break;
      case ERR_LONGS_NOT_ALLOWED                      : Print("Longs are not allowed - Check the expert properties"); break;
      case ERR_SHORTS_NOT_ALLOWED                     : Print("Shorts are not allowed - Check the expert properties"); break;
      case ERR_OBJECT_ALREADY_EXISTS                  : Print("Object exists already"); break;
      case ERR_UNKNOWN_OBJECT_PROPERTY                : Print("Unknown object property"); break;
      case ERR_OBJECT_DOES_NOT_EXIST                  : Print("Object does not exist"); break;
      case ERR_UNKNOWN_OBJECT_TYPE                    : Print("Unknown object type"); break;
      case ERR_NO_OBJECT_NAME                         : Print("No object name"); break;
      case ERR_OBJECT_COORDINATES_ERROR               : Print("Object coordinates error"); break;
      case ERR_NO_SPECIFIED_SUBWINDOW                 : Print("No specified subwindow"); break;
      case ERR_SOME_OBJECT_ERROR                      : Print("Some error in object function"); break;
   }
}